package MN::Engine; # class to manage MyNews engine

use strict;

use DBI;

my $current;

sub getCurrent # get current engine
{
    return $current;
}

sub new # constructor
{
    my $class_name = shift;
    my $conf_file = shift || $ENV{MNCONF} || "/etc/mynews.conf";

    my $self = {};
    
    bless $self, $class_name;

    $self->{conf_file} = $conf_file;

    $self->readConf();

    $self->initBackends();

    $self->connectDB();

    $current = $self;

    return $self; # new Engine object
}

sub readConf # read configuration file
{
    my $self = shift;
    
    $self->{conf} = {};
    $self->{conf}->{backends} = {};

    open(CONF, $self->{conf_file});
    my $line;
    my $conf = $self->{conf};
    while ($line = <CONF>)
    {
	if ($line =~ /^\s*\#$/)
	{
	    next;
	}
	elsif ($line =~ /^\s*backend\s*(.*)$/)
	{
	    $self->{conf}->{backends}->{$1} = {};
	    $conf = $self->{conf}->{backends}->{$1};
	}
	elsif ($line =~ /^\s*end\s*backend\s*$/)
	{
	    $conf = $self->{conf};
	}
	if ($line =~ /^\s*(\S+)\s*=\s*(.*)$/)
	{
	    $conf->{$1} = $2;
	}
    }
}

sub initBackends # init the database backends
{
    my $self = shift;

    my @drivers = DBI->available_drivers();
    my $driver;
    my $drv;
    my $backend;
    my $conf;
    my $found;
    $self->{backends} = {};
    while (($backend,$conf) = each (%{$self->{conf}->{backends}}))
    {
	$self->{backends}->{$backend} = {};
	if ($conf->{type} eq "pgsql")
	{
	    $driver = 'Pg';
	    $self->{backends}->{$backend}->{fsep} = '"';
	    $self->{backends}->{$backend}->{tsep} = '"';
	    $self->{backends}->{$backend}->{dsn} = "dbi:Pg:dbname=$conf->{database}";
	}
	elsif ($conf->{type} eq "mysql")
	{
	    $driver = 'mysql';
	    $self->{backends}->{$backend}->{fsep} = '`';
	    $self->{backends}->{$backend}->{tsep} = '`';
	    $self->{backends}->{$backend}->{concatfn} = "concat";
	    $self->{backends}->{$backend}->{dsn} = "dbi:mysql:database=$conf->{database}";
	}
	elsif ($conf->{type} eq "oracle")
	{
	    $driver = 'Oracle';
	    $self->{backends}->{$backend}->{fsep} = '"';
	    $self->{backends}->{$backend}->{tsep} = '';
	    $self->{backends}->{$backend}->{nolimit} = 1;
	    $self->{backends}->{$backend}->{dsn} = "dbi:Oracle:$conf->{database}";
	}
	else
	{
	    die "Unknown backend type $conf->{type}";
	}

	$found = 0;
	foreach $drv (@drivers)
	{
	    if ($drv eq $driver)
	    {
		$found = 1;
		last;
	    }
	}
	if (!$found)
	{
	    die "DBD::$driver perl module not installed";
	}
    }
}

sub sqlConcat # return string concatenation for sql query
{
    my $self = shift;
    my $backend = shift; # selected backend

    if ($self->{backends}->{$backend}->{concatfn})
    {
	return $self->{backends}->{$backend}->{concatfn}.'('.join(',', @_).')';
    }
    else
    {
	return join('||', @_);
    }
}

sub connectDB # connect to the databases
{
    my $self = shift;
    
    my $backend;
    my $be;
    while (($backend, $be) = each (%{$self->{backends}}))
    {
	$self->{backends}->{$backend}->{dbh} = DBI->connect($be->{dsn}, $self->{conf}->{backends}->{$backend}->{user}, $self->{conf}->{backends}->{$backend}->{pass});
    }
}

sub disconnectDB # disconnect from the databases
{
    my $self = shift;
    
    my $backend;
    my $be;
    while (($backend, $be) = each (%{$self->{backends}}))
    {
	if ($be->{dbh})
	{
	    $be->{dbh}->disconnect();
	}
    }
}

sub getDBVersion # get the DBMS version
{
    my $self = shift;
    my $backend = shift; # selected backend
    
    my $query;
    if ($self->{conf}->{backends}->{$backend}->{type} eq "oracle")
    {
	$query = 'select * from v$version';
    }
    else
    {
	$query = 'select version()';
    }
    my $sth = $self->{backends}->{$backend}->{dbh}->prepare($query);
    if (!$sth->execute())
    {
	_c_("Error in $backend query [$query]\n");
    }
    my @row = $sth->fetchrow_array();
    my $version = $row[0];

    if ($self->{conf}->{backends}->{$backend}->{type} eq "pgsql" && $version =~ /^PostgreSQL\s+(\S+)/)
    {
	$version = $1;
    }
    elsif ($self->{conf}->{backends}->{$backend}->{type} eq "oracle" && $version =~ /(Release|Version)\s+(\S+)/)
    {
	$version = $2;
    }
    return $version;
}

sub getKernelVersion # get the Linux kernel version
{
    my $self = shift;
    
    my $version = "";

    open(PROC, "/proc/version");
    my $line = <PROC>;
    close(PROC);

    if ($line =~ /([\d\.]+)/)
    {
	$version = $1;
    }
    return $version;
}

sub getDistribVersion # get the Linux distribution version
{
    my $self = shift;
    
    my $version = "";

    open(REL, "/etc/redhat-release");
    my $line = <REL>;
    close(REL);

    if ($line)
    {
	chomp($line);
	$line =~ s/linux //gi;
	$line =~ s/release //gi;
	$version = $line;
    }
    return $version;
}

sub getCPUInfo # get the CPU information
{
    my $self = shift;
    
    my $model = "";
    my $mhz = "";

    my $line;

    open(PROC, "/proc/cpuinfo");
    while ($line = <PROC>)
    {
	chomp($line);
	if ($line =~ /model name\s*:\s*(.*)/)
	{
	    $model = $1;
	    $model =~ s/\([^\)]*\)//g;
	}
	elsif ($line =~ /cpu MHz\s*:\s*(\d*)/)
	{
	    $mhz = " $1 Mhz";
	}
    }
    close(PROC);

    return $model.$mhz;
}

sub getMemInfo # get the CPU information
{
    my $self = shift;
    
    my $ko = "";

    my $line;

    open(PROC, "/proc/meminfo");
    while ($line = <PROC>)
    {
	chomp($line);
	if ($line =~ /MemTotal:\s*(\d*)/)
	{
	    $ko = $1;
	}
    }
    close(PROC);

    my $mo = int(.5 + ($ko / 1024));
    return "$mo Mo";
}

sub getWWWVersion # get the MyNews website version
{
     my $self = shift;
   
     if ($self->{conf}->{webdir})
     {
	 open(VERS, "$self->{conf}->{webdir}/version.txt");
	 my $line = <VERS>;
	 chomp($line);
	 close(VERS);
	 return $line;
     }
     return "";
}

sub isHidden # check wether a visual is displayed or not for public site
{
    my $self = shift;
    my $visual = shift; # visual to test

    my $dispcodes = MN::User->getDispcodes();

    if (defined $self->{conf}->{pubdisplay})
    {
	if ($self->{conf}->{pubdisplay} & $dispcodes->{$visual})
	{
	    return ""; # "" if dislayed
	}
	else
	{
	    return "hidden";
	}
    }
    else
    {
	return ""; # always dislayed
    }
}

sub getNPP # get default NewsPerPage value for public site
{
    my $self = shift;
	
    return $self->{conf}->{pubnpp} || 10;
}

sub DESTROY # destructor
{
    my $self = shift;

    $self->disconnectDB();
}

1;

__END__

=pod

=head1 NAME

MN::Engine - Engine to manage MyNews configuration and database connection.

=head1 SYNOPSIS

 use strict;
 use MN 1.0;

 my $mne = new MN::Engine();
 ...
 my $mne = MN::Engine->getCurrent();

=head1 DESCRIPTION

MyNews engine is used to read MyNews configuration file and to initialize database connection. The engine creation must be the first operation of any script that use MyNews.

The current engine can be accessed at any time by using the B<getCurrent> class method.

At the end of the script, when the current engine is delete, the database connection is closed automatically.

=over 4

=item new ([$conf_file])

Engine constructor. Optionnal parameter can be set to read an alternate configuration file (/etc/mynews.conf by default).

=item getDBVersion ()

Get the current database manager version. Example for PostgreSQL: 7.3.4

=item getKernelVersion ()

Get the Linux kernel that runs on the MyNews server. Example: 2.4.22

=item getDistribVersion ()

Get the Linux distribution that runs on the MyNews server. Example: Mandrake 9.2

=item getCPUInfo ()

Get information about the CPU of the MyNews server (Linux only). Example: Pentium 4 2665 Mhz

=item getMemInfo ()

Get information about the RAM of the MyNews server (Linux only). Example: 756 Mo

=item isHidden ($visual)

Return "hidden" if a visual is hidden for the MyNews public web site.

=item getNPP ()

Return the default number of news per page for the MyNews public web site.

=head1 SEE ALSO

L<MN::CGI>

=cut
