package MN::Object; # base class to manage database persistant objects

use strict;

use MN::Engine;
use MN::CGI;

my %extrameth = ();

sub getpmx # get .pmx file for a module
{
    my $class_name = shift;
    my $class = shift;
    my @pmx = ();

    use Config();
    my $path =  $Config::Config{installsitelib};
    $class =~ s/::/\//g;
    opendir(DIR, "$path/$class");
    my $file;
    while ($file = readdir(DIR))
    {
	if ($file =~ /\.pmx$/)
	{
	    push @pmx, "$path/$class/$file";
	}
    }
    closedir(DIR);
    return @pmx;
}

sub regmeth # register an extra method
{
    my $class_name = shift;
    my $class = shift;
    my $key = shift;
    my $meth = shift;

    if (!defined($extrameth{$class})) {$extrameth{$class} = {}};
    if (!defined($extrameth{$class}->{$key})) {$extrameth{$class}->{$key} = {}};
    $extrameth{$class}->{$key}->{$meth} = 1;
}

sub extrameths # call extra methods
{
    my $self = shift;
    my $class = ref($self) || $self;
    my $key = shift;

    my $meth;
    foreach $meth (keys(%{$extrameth{$class}->{$key}}))
    {
	$self->$meth(@_);
    }
}

sub new # constructor
{
    my $class_name = shift;

    my $self = {};
    
    bless $self, $class_name;

    return $self; # new Object object
}

sub getClass # get database table for the class
{
    return ""; # "" (virtual mother class)
}

sub getBackend # get database backend for the class
{
    return "default"; # "default" (virtual mother class)
}

sub getKey # get database key for the class
{
    return ""; # "" (virtual mother class)
}

sub getId # get database key id for the object
{
    my $self = shift;

    my @keys = split(/,/, $self->getKey());
    if (scalar(@keys) > 1)
    {
	return map {$self->{$_}} @keys; # object database key ids table
    }
    else
    {
	return $self->{$keys[0]}; # object database key id
    }
}

sub getFields # get database fields for the class
{
    return {}; # empty hash table (virtual mother class)
}

sub dbValue # convert a string into a database value
{
    my $self = shift;
    my $value = shift; # value to convert
    my $type = shift; # type of value (number, string...)
    
    if ($type eq "number")
    {
	return $value || 0; # "value" for number type
    }
    elsif($type eq "bool")
    {
	if(!$value)
	{
	    return "'f'";
	}
	elsif(($value eq "t") || ($value eq "f"))
	{
	    return "'".$value."'";
	}
	else
	{
	    return $value ? "'t'" : "'f'";
	}
    }
    elsif ($type eq "date" && !$value)
    {
	return "NULL";
    }
    else
    {
	my $mne = MN::Engine->getCurrent(); # MyNews Engine
	my $backend = $self->getBackend();
	return $mne->{backends}->{$backend}->{dbh}->quote($value); # "'value'" for other types
    }
}

sub getObjects # return list of db objects
{
    my $self = shift;
    my $filter = shift; # search condition
    my $order = shift; # order condition (optional)
    my $limit = shift; # to limit selection to a fixed number of objects (optional)
    my $offset = shift; # to jump to a fixed position in the query (optional)

    my $class = ref($self) || $self;
    my $mne = MN::Engine->getCurrent(); # MyNews Engine
    my $backend = $self->getBackend();

    my @objects;
    my $object;
    my $fsep = $mne->{backends}->{$backend}->{fsep};
    my $tsep = $mne->{backends}->{$backend}->{tsep};
    my $nolimit = $mne->{backends}->{$backend}->{nolimit};

    my @fields = keys(%{$self->getFields()});
    my $query = 'select '.$fsep.join($fsep.','.$fsep, @fields).$fsep.' from ';
    $query .= $tsep.$self->getClass().$tsep;
    if ($filter)
    {
	$filter =~ s//$fsep/g;
	$filter =~ s//$tsep/g;
	$query .= " where $filter";
    }
    if ($order)
    {
	$order =~ s//$fsep/g;
	$order =~ s//$tsep/g;
	$query .= " order by $order";
    }
    if ($limit && !$nolimit)
    {
	$query .= " limit $limit";
    }
    if ($offset && !$nolimit)
    {
	$query .= " offset $offset";
    }

    my $sth = $mne->{backends}->{$backend}->{dbh}->prepare($query);
    if (!$sth->execute())
    {
	_c_("Error in query [$query]\n");
    }
    my @row;
    my $i;
    my $j;
    my $field;
    if ($offset && $nolimit)
    {
	for ($i = 0; $i < $offset; $i++)
	{
	    $sth->fetchrow_array();
	}
    }
    $j = 0;
    while (@row = $sth->fetchrow_array())
    {
	if ($nolimit && $limit && $j >= $limit)
	{
	    last;
	}
	$object = eval("use $class; new $class();");
	$i = 0;
	foreach $field (@fields)
	{
	    $object->{$field} = $row[$i++];
	}
	push @objects, $object;
	$j++;
    }
    return @objects; # corresponding db objects in a table
}

sub getObject # return db object by key
{
    my $self = shift;
    my @keys = @_; # keys of the object
    
    if (scalar(@keys))
    {
	my $keynames = $self->getKey();
	my $fields = $self->getFields();

	my $keyname;
	my @conds;
	my $cond;
	my $key;
	foreach $keyname (split(/,/, $keynames))
	{
	    $key = shift @keys;
	    if (!defined($key))
	    {
		return undef; # undef if missing key
	    }
	    push @conds, ''.$keyname.'='.$self->dbValue($key, $fields->{$keyname});
	}
	$cond = join(' and ', @conds);
	my @objects = $self->getObjects($cond);
	return $objects[0]; # the corresponding db object
    }
    else
    {
	return undef; # undef if db object not found
    }
}

sub getObjectsCount # return the number of objects that match a condition
{
    my $self = shift;
    my $filter = shift; # condition to match

    my $class = ref($self) || $self;
    my $mne = MN::Engine->getCurrent(); # MyNews Engine
    my $backend = $self->getBackend();
    my $fsep = $mne->{backends}->{$backend}->{fsep};
    my $tsep = $mne->{backends}->{$backend}->{tsep};

    my $query = 'select count(*) from ';
    $query .= $tsep.$self->getClass().$tsep;
    if ($filter)
    {
	$filter =~ s//$fsep/g;
	$filter =~ s//$tsep/g;
	$query .= " where $filter";
    }

    my $sth = $mne->{backends}->{$backend}->{dbh}->prepare($query);
    if (!$sth->execute())
    {
	_c_("Error in query [$query]\n");
    }
    my @row = $sth->fetchrow_array();

    return $row[0]; # number of objects that match the condition
}

sub add # add object to the database
{
    my $self = shift;

    my $mne = MN::Engine->getCurrent(); # MyNews Engine
    my $backend = $self->getBackend();
    my $fsep = $mne->{backends}->{$backend}->{fsep};
    my $tsep = $mne->{backends}->{$backend}->{tsep};

    my $fields = $self->getFields();
    my $query = 'insert into '.$tsep.$self->getClass();
    $query .= $tsep.' ('.$fsep.join($fsep.','.$fsep, keys(%{$fields})).$fsep.') values ';
    my $field;
    my $type;
    $fsep = '(';
    while (($field,$type) = each(%{$fields}))
    {
	my $val = $self->dbValue($self->{$field}, $type);
	$query = join("",$query,$fsep,$val);
	$fsep = ',';
    }
    $query .= ")";

    my $sth = $mne->{backends}->{$backend}->{dbh}->prepare($query);
    if (!$sth->execute())
    {
	_c_("Error in query [$query]\n");
    }

    return 0; # 0 if OK
}

sub modify # modify object in the database
{
    my $self = shift;
    my @keyvals = @_; # optional old key val is changed in the object

    if (!scalar(@keyvals))
    {
	@keyvals = $self->getId();
    }

    my $mne = MN::Engine->getCurrent(); # MyNews Engine
    my $backend = $self->getBackend();
    my $fsep = $mne->{backends}->{$backend}->{fsep};
    my $tsep = $mne->{backends}->{$backend}->{tsep};

    my $keynames = $self->getKey();
    my $fields = $self->getFields();
    my $keyname;
    my @conds;
    my $cond;
    my $key;
    foreach $keyname (split(/,/, $keynames))
    {
	$key = shift @keyvals;
	if (!defined($key))
	{
	    return undef; # undef if missing key
	}
	push @conds, $fsep.$keyname.$fsep.'='.$self->dbValue($key, $fields->{$keyname});
    }
    $cond = join(' and ', @conds);
    my $query = 'update '.$tsep.$self->getClass();
    $query .= $tsep.' set';
    my $field;
    my $type;
    my $sep2 = ' ';
    while (($field,$type) = each(%{$fields}))
    {
	$query .= $sep2.$fsep.$field.$fsep.'='.$self->dbValue($self->{$field}, $type);
	$sep2 = ',';
    }
    $query .= ' where '.$cond;

    my $sth = $mne->{backends}->{$backend}->{dbh}->prepare($query);
    if (!$sth->execute())
    {
	_c_("Error in query [$query]\n");
    }

    return 0; # 0 if OK
}

sub delete # delete object from the database
{
    my $self = shift;

    my $mne = MN::Engine->getCurrent(); # MyNews Engine
    my $backend = $self->getBackend();
    my $fsep = $mne->{backends}->{$backend}->{fsep};
    my $tsep = $mne->{backends}->{$backend}->{tsep};

    my $keynames = $self->getKey();
    my $fields = $self->getFields();
    my @keyvals = $self->getId();
    my $keyname;
    my @conds;
    my $cond;
    my $key;
    foreach $keyname (split(/,/, $keynames))
    {
	$key = shift @keyvals;
	if (!defined($key))
	{
	    return undef; # undef if missing key
	}
	push @conds, $fsep.$keyname.$fsep.'='.$self->dbValue($key, $fields->{$keyname});
    }
    $cond = join(' and ', @conds);
    my $query = 'delete from '.$tsep.$self->getClass();
    $query .= $tsep.' where '.$cond;

    my $sth = $mne->{backends}->{$backend}->{dbh}->prepare($query);
    if (!$sth->execute())
    {
	_c_("Error in query [$query]\n")
    }

    return 0; # 0 if OK
}

sub modifyAttribute # set object attribute and modify it in the database
{
    my $self = shift;
    my $attr = shift; # object attribute
    my $value = shift; # attribute value

    my $mne = MN::Engine->getCurrent(); # MyNews Engine
    my $backend = $self->getBackend();
    my $fsep = $mne->{backends}->{$backend}->{fsep};
    my $tsep = $mne->{backends}->{$backend}->{tsep};

    my $keynames = $self->getKey();
    my $fields = $self->getFields();
    my @keyvals = $self->getId();
    my $keyname;
    my @conds;
    my $cond;
    my $key;

    $self->{$attr} = $value;

    foreach $keyname (split(/,/, $keynames))
    {
	$key = shift @keyvals;
	if (!defined($key))
	{
	    return undef; # undef if missing key
	}
	push @conds, $fsep.$keyname.$fsep.'='.$self->dbValue($key, $fields->{$keyname});
    }
    $cond = join(' and ', @conds);
    my $query = 'update '.$tsep.$self->getClass();
    $query .= $tsep.' set '.$fsep.$attr.$fsep.'='.$self->dbValue($value, $fields->{$attr});
    $query .= ' where '.$cond;

    my $sth = $mne->{backends}->{$backend}->{dbh}->prepare($query);
    if (!$sth->execute())
    {
	_c_("Error in query [$query]\n");
    }

    return 0; # 0 if OK
}

sub updateFrom # update an object from hash table
{
    my $self = shift;
    my $atts = shift; # hash table of attribute values
    my %atts = %{$atts};

    my $key;
    my $val;
    while (($key, $val) = each(%atts))
    {
	if ($self->{$key} ne "" ||  $val ne "")
	{
	    $self->{$key} = $val;
	}
    }
}

sub execSQL # execute SQL query in the class database
{
    my $self = shift;
    my $query = shift; # query to exec

    my $mne = MN::Engine->getCurrent(); # MyNews Engine
    my $backend = $self->getBackend();
    my $fsep = $mne->{backends}->{$backend}->{fsep};
    my $tsep = $mne->{backends}->{$backend}->{tsep};

    $query =~ s//$fsep/g;
    $query =~ s//$tsep/g;

    my $sth = $mne->{backends}->{$backend}->{dbh}->prepare($query);
    if (!$sth->execute())
    {
	_c_("Error in query [$query]\n");
	return 1;
    }
    return 0;
}

1;

__END__

=pod

=head1 NAME

MN::Object - MyNews base class to manage database persistant objects.

=head1 SYNOPSIS

 use strict;
 use MN 1.0;

 my $mne = new MN::Engine();
 my $cgi = new MN::CGI();
 # get the 10 more recent news
 my @newss = MN::News->getObjects("", "date desc, crtime desc", 10) 
 ...

=head1 DESCRIPTION

MyNews Object is the mother class of all object stored in the database. MN::Object is not used directly. It only defines virtual methods that are redefined in subclasses.

=over 4

=head2 Class description methods

=item getClass ()

Return the name of the database table used to store objects of the class. For example, the B<getClass> method of the class B<MN::User> returns "mnuser".

=item getBackend ()

Return the name of the database backend for the class. Default value = "default". Can be overridden to store some classes in another backend.

=item getKey ()

Return the primary key of the class. For example, the B<getKey> method of the class B<MN::User> returns "login". 

Multiple fields primary key must be returned separated bt commas. For example, the B<getKey> method of the class B<MN::UserGroup> returns "login,groupid". 

=item getFields ()

Return the fields of the class to store in the database. The return value is a hash table (fieldname => fieldtype). For example, the B<getFields> method of the class B<MN::User> returns:

       ("login" => "string",
	"password" => "string",
	"mail" => "string",
	"lang" => "string",
	"theme" => "string",
	"firstname" => "string",
	"lastname" => "string",
	"display" => "number",
	"newsperpage" => "number")

=head2 Select objects from the database

=item getObject ($key, [$key2...])

Select an object by its primary key. If primary key has multiple fields, all keys must be passed to the method. For example to select the "slashdot" template:

 my $tmpl = MN::Template->getObject("slashdot");

=item getObjects ($filter, [$order], [$limit], [$offset])

Select several objects of the class from the database and return them as a table.

$filter is the SQL condition that selected objects must match. Field name must be surrounded by  characters (international current symbol). For example to select news of the "slashdot" template, you can write:

 my @nnews = MN::News->getObjects("template = 'slashdot'")

$order is the sort order for the selected object. For example to sort by date (recent to old):

 my @nnews = MN::News->getObjects("template = 'slashdot'", "date desc")

$limit is the maximun number of selected objects. For example to select the 10 more recent news:

 my @nnews = MN::News->getObjects("template = 'slashdot'", "date desc", 10)

$offset is used to select the next objects when $limit is less than the total number of objects. For example to select the next 10 news after the 10 more recent news:

 my @nnews = MN::News->getObjects("template = 'slashdot'", "date desc", 10, 10)

=item getObjectsCount ($filter)

Return the number of objects that match a condition.

$filter is the SQL condition that selected objects must match. For example to get the number of news of the "slashdot" template:

 my $nbn =  MN::News->getObjectsCount("template = 'slashdot'");

=head2 Write objects in the database

=item add ()

Insert an object in the database. Primary key must be set before calling this method.

=item modify ([$oldkey...])

Modify an object in the database. Object must be present in the database before calling this method.

The optional $oldkey parameters is useful only to modify the primary key of the object.

=item modifyAttribute ($field, $value)

Change the value of one field of the object and save it in the database. Object must be present in the database before calling this method.

=item delete ()

Delete an object in the database. Object must be present in the database before calling this method.

=head2 Module extension

=item getpmx ($class)

Get the list of all .pmx files that extend the class.

=item getmeth ($class, $key, $meth)

Register a method called $meth for the class $class under the key $key. This method will be called by extrameths($key).

=item extrameths ($key)

Call extension methods registered under the key $key on the current object.

=head2 Misc

=item updateFrom ($values)

Modify the value of the fields of the object according to a hash table of value. Modifications are not saved in the database.

$values is a hash table (fieldname => value).

=item execSQL ($query)

Execute a SQL query in the database. Used to speed up some massive operations.

$query is the SQL query to execute. Field names are surrounded by  characters and table names are surrounded by  characters. For example to delete all news of slashdot templates:

 MN::News->execSQL("delete from mnnews where template = 'slashdot'")

=item getId ()

Return the primary key of an object. If the primary key has multiple fields, the method returns a table of key values.

=head1 SEE ALSO

L<MN::Engine>, L<MN::CGI>
