package MN::Topic; # class to manage MyNews topics

use strict;

@MN::Topic::ISA = qw(MN::Object);

use MN::Object;
use HTTP::Lite;

my %current = ();

sub getCurrent # get current topic hash
{
    return \%current;
}

sub new # constructor
{
    my $class_name = shift;

    my $self = new MN::Object();
    
    bless $self, $class_name;

    return $self; # new Topic object
}
sub getClass # get database table for the class 
{
    return "mntopic"; # "mntopic"
}

sub getKey # get database key for the class
{
    return "id"; # "id"
}

sub getFields # get database fields for the class
{
    return { # fields hash table
	"id" => "string",
	"name" => "string",
	"iconfile" => "string",
    };
}

sub getTopicId # get the topic id (without template name)
{
    my $self = shift;

    my $id = $self->{id};
    $id =~ s/^[^_]+_//;
    return $id; # the topic id
}

sub getTemplateId # get the template id of the topic
{
    my $self = shift;

    my $id = $self->{id};
    if ($id =~ /^([^_]+)/)
    {
	return $1; # the template id
    }
    else
    {
	return $id; # id on error
    }
}

sub getTemplate # get the news template
{
    my $self = shift;

    if (!$self->{template} && $self->{id} =~ /^([^_]+)/)
    {
	$self->{template} = MN::Template->getTemplate($1);
    }
    return $self->{template};
}

sub getTopic # get news topic by its name
{
    my $self = shift;
    my $name = shift; # name of the topic

    if (!$name)
    {
	return undef;
    }

    if (!$current{$name})
    {
	$current{$name} = MN::Topic->getObject($name);
    }
    return $current{$name};
}

sub getCode # get HTML code for the topic;
{
    my $self = shift;
    my $css = shift;
    my $reload = shift; # 1 to add reload button
    my $test = shift; # 1 1 to display topic in test mode

    my $code = "<div class=\"newstopic$css\" title=\"$self->{name}\">\n";
    my $imgdir;
    my $imgurl;
    my $url;
    my $mne = MN::Engine::getCurrent(); # MyNews Engine

    if ($reload || $test)
    {
	if ($self->{iconfile})
	{
	    my $tmpl = $self->getTemplate();
	    if ($tmpl->{link_icon})
	    {
		$url = sprintf($tmpl->{link_icon}, $self->{iconfile});
	    }
	}
    }
    elsif ($mne->{conf}->{weburl})
    {
	$url = $mne->{conf}->{weburl}.'/topic.cgi?id='.$self->{id}
    }

    if ($mne->{conf}->{webdir})
    {
	$imgdir = "$mne->{conf}->{webdir}/img/topics";
    }
    if ($url)
    {
	$imgurl = "$mne->{conf}->{weburl}/img/topics";
	$code .= '<a href="';
	if ($test)
	{
	    $code .= $mne->{conf}->{weburl}.'/topic.cgi?id='.$self->{id}
	}
	else
	{
	    $code .= $url;
	}
	$code .= '" class="newstopic'.$css.'">'."\n";
    }
    if ($test && $url)
    {
	$code .= '<img  class="newstopicimg'.$css.'" src="'.$url.'" border=0 alt="'.$self->{name}.'" title="'.$self->{name}.'">'."\n";
    }
    elsif ($imgurl && $imgdir && -e "$imgdir/$self->{id}.png")
    {
	$code .= '<img  class="newstopicimg'.$css.'" src="'.$imgurl.'/'.$self->{id}.'.png" border=0 alt="'.$self->{name}.'" title="'.$self->{name}.'">'."\n";
    }
    else
    {
	$code .= $self->{name}."\n";
    }
    if ($url)
    {
	$code .= "</a>\n";
	if ($reload)
	{
	    $code .= '<a href="topics.cgi?reload='.$self->{id}.'">'."\n";
	    $code .= '<div class="reload" title="'.MN::__("Reload icon from website").'">'."\n";
	    $code .= "</div>\n";
	    $code .= "</a>\n";
	    $code .= '<a href="topics.cgi?custom='.$self->{id}.'">'."\n";
	    $code .= '<div class="custom" title="'.MN::__("Customize icon").'">'."\n";
	    $code .= "</div>\n";
	    $code .= "</a>\n";
	}
    }
    $code .= "</div>\n";

    return $code; # topic HTML code
}

sub getTopicIcon # extract topic icon from template website
{
    my $self = shift;

    my $mne = MN::Engine::getCurrent(); # MyNews Engine
    my $imgdir;
    if ($mne->{conf}->{webdir})
    {
	$imgdir = "$mne->{conf}->{webdir}/img/topics";
    }
    if (!$imgdir)
    {
	return 0;
    }
    my $tmpl = $self->getTemplate();
    if ($self->{iconfile} && $tmpl->{link_icon})
    {
	my $http = new HTTP::Lite();
	$http->add_req_header("Accept-Language", $tmpl->{lang});
	$http->add_req_header("User-Agent", "MyNews $MN::VERSION");
	my $req = $http->request(sprintf($tmpl->{link_icon}, $self->{iconfile}));
	if ($req)
	{
	    open(IMG, ">$imgdir/$self->{id}.png");
	    print IMG $http->body();
	    close(IMG);
	    chmod(0664, "$imgdir/$self->{id}.png");
	    return 0;
	}
	else
	{
	    my $msg = $http->status_message();
	    $msg =~ s/[\r\n]+//g;
	    return $msg;
	}	
    }
    else
    {
	return 0;
    }
}

sub delete # database destructor
{
    my $self = shift;

    # delete icon
    my $mne = MN::Engine::getCurrent(); # MyNews Engine
    if ($mne->{conf}->{webdir})
    {
	my $iconfile = "$mne->{conf}->{webdir}/img/topics/$self->{id}.png";
	if (-e $iconfile)
	{
	    unlink($iconfile);
	}
    }

    $self->SUPER::delete();
}

1;

__END__

=pod

=head1 NAME

MN::Topic - MyNews class to manage topics.

=head1 SYNOPSIS

 use strict;
 use MN 1.0;

 my $mne = new MN::Engine();
 my $cgi = new MN::CGI();
 my $topic = MN::Topic->getObject("slashdot_222");
 ...

=head1 DESCRIPTION

MyNews Topic objects store informations about template topics.

MN::Topic inherits from MN::Object, that means that topic objects are stored in database.

=over 4

=head2 Fields

=item id : string

Topic id (primary key). 

=item name : string

Topic name (displayed as alternate text to topic icon).

=item iconfile : string

Name of the icon file of the topic in the website.

=head2 Topic information

=item getTopicId ()

Return the topic id without template prefix. For example: "222"

=item getTemplateId ()

Return the template id of the topic. For example: "slashdot"

=item getTemplate ()

Return template object corresponding to the topic. Avoid database access if the template object has been already selected.

=item getTopic ($id)

Return topic object by its id. Avoid database access if the topic object has been already selected.

=item getCode ([$css], [$reload], [$test])

Return the HTML code to display the topic in the MyNews website.

$css is the suffix to add to the topic CSS class.

If $reload is set to 1, reload and custom buttons is added to HTML code.

If $test is set to 1, display the topic in test mode (use remote icon).

=head2 Topic extraction

=item getTopicIcon ()

Extract icon of the topic.

=head1 SEE ALSO

L<MN::Object>, L<MN::Template>, L<MN::News>

=cut
