package MN::User; # class to manage MyNews users

use strict;

@MN::User::ISA = qw(MN::Object);

use MN::Object;
use MN::UserGroup;
use MN::Template;

my %dispcodes = 
    (
     "Date" => 1,
     "Time" => 2,
     "Century" => 4,
     "Seconds" => 8,
     "Flag" => 16,
     "Navigation bar" => 32,
     "Console" => 64,
     );

BEGIN 
{ 
    use MN::Object;
    my $ext;
    foreach $ext (MN::Object->getpmx("MN::User")) 
    { 
	require $ext; 
    }
}

sub new # constructor
{
    my $class_name = shift;

    my $self = new MN::Object();
    
    bless $self, $class_name;

    return $self; # new User object
}

sub getClass # get database table for the class 
{
    return "mnuser"; # "mnuser"
}

sub getKey # get database key for the class
{
    return "login"; # "login"
}

sub getFields # get database fields for the class
{
    return { # fields hash table
	"login" => "string",
	"password" => "string",
	"mail" => "string",
	"lang" => "string",
	"theme" => "string",
	"firstname" => "string",
	"lastname" => "string",
	"display" => "number",
	"newsperpage" => "number",
    };
}

sub getDispcodes # get dispcodes hash table
{
    return \%dispcodes;
}

sub getSubTemplates # get the templates subscribed by the user
{
    my $self = shift;
    my $ignoreactive = shift; # 1 to ignore active flag 

    if (!$self->{tmpls})
    {
	my @sbs = MN::Subscr->getObjects("user = '$self->{login}'", "template");
	my $sb;
	my @tmpls = ();
	my $tmpl;
	foreach $sb (@sbs)
	{
	    $tmpl = MN::Template->getTemplate($sb->{template});
	    if ($tmpl && ($ignoreactive || $tmpl->{active}))
	    {
		push @tmpls, $tmpl;
	    }
	}
	$self->{tmpls} = \@tmpls;
    }
    return $self->{tmpls};
}

sub getUnsubTemplates # get the templates not subscribed by the user
{
    my $self = shift;
    
    if (!$self->{utmpls})
    {
	my @sbs = MN::Subscr->getObjects("user = '$self->{login}'", "template");
	my $cond = "active = 1";
	if (scalar(@sbs))
	{
	    $cond .= " and id not in ('".join("','", map {$_->{template}} @sbs)."')";
	}
	my @tmpls = MN::Template->getObjects($cond, "id");
	my $tmpl;
	foreach $tmpl (@tmpls)
	{
	    $tmpl = MN::Template::getCurrent()->{$tmpl->{id}} = $tmpl;
	}
	$self->{utmpls} = \@tmpls;
    }
    return $self->{utmpls};
}

sub adminSelectorCode # return HTML code for the admin selector
{
    my $self = shift;
    my $name = shift || "admin"; # optional selector name ("admin" by default)

    my $code = "";
    my $admin = MN::UserGroup->getObject($self->{login}, "admin");

    if ($ENV{REMOTE_USER} eq $self->{login})
    {
	if ($admin)
	{
	    $code .=  MN::__("Yes")."\n";
	}
	else
 	{
	    $code .=  MN::__("No")."\n";
	}
	$code .= '<input type="hidden" name="'.$name.'" value="'.(defined $admin).'">'."\n";
    }
    else
    {
	$code .= '<select name="'.$name.'">'."\n";
	$code .= '<option value="1"';
	if ($admin)
	{
	    $code .= " selected";
	}
	$code .= ">".MN::__("Yes")."\n";
	$code .= '<option value="0"';
	if (!$admin)
	{
	    $code .= " selected";
	}
	$code .= ">".MN::__("No")."\n";
	$code .= "</select>\n";
    }
    $code .= '<input type="hidden" name="old'.$name.'" value="'.(defined $admin).'">'."\n";

    return $code;
}

sub isAdmin # return "Yes" if user has admin privilege ("No" otherwise)
{
    my $self = shift;
    my $admin = MN::UserGroup->getObject($self->{login}, "admin");

    return $admin ? MN::__("Yes") : MN::__("No");
}

sub setAdmin # set or unset admin privilege to the user
{
    my $self = shift;
    my $value = shift; # # 1 to set, 0 to unset

    my $admin = MN::UserGroup->getObject($self->{login}, "admin");
    if ($value)
    {
	if (!$admin)
	{
	    $admin = new MN::UserGroup();
	    $admin->{login} = $self->{login};
	    $admin->{groupid} = "admin";
	    $admin->add();
	}
    }
    else
    {
	if ($admin)
	{
	    $admin->delete();
	}
    }
}

sub delete # database destructor
{
    my $self = shift;

    # delete user/group objects
    my @usgrs = MN::UserGroup->getObjects("login='$self->{login}'");
    my $usgr;
    foreach $usgr (@usgrs)
    {
	$usgr->delete();
    }

    # delete user subscriptions
    my @usbs = MN::Subscr->getObjects("user='$self->{login}'");
    my $usb;
    foreach $usb (@usbs)
    {
	$usb->delete();
    }

    $self->SUPER::delete();
}

sub displaySelectorCode # return HTML code for the display selector
{
    my $self = shift;
    my $submit = shift; # 1 to submit on value change
    my $name = shift || "display"; # optional selector name ("display" by default)

    my $code = "";
    $code .= '<select name="'.$name.'" multiple';
    if ($submit)
    {
	$code .= ' onchange="form.submit();"';
    }
    $code .= ">\n";
    my $key;
    my $admin = 0;
    if ($self->isAdmin() eq MN::__("Yes"))
    {
	$admin = 1;
    }
    foreach $key (sort(keys(%dispcodes)))
    {
	if (!$admin && $key eq "Console") { next; }

	$code .= '<option value="'.$dispcodes{$key}.'"';
	if ($self->{display} & $dispcodes{$key})
	{
	    $code .= " selected";
	}
	$code .= '>'.MN::__($key)."\n";
    }
    $code .= "</select>\n";

    return $code;
}

sub setDisplay # set user display value from single or multiple values
{
    my $self = shift;
    my $values = shift; # values separated by \0

    my @values = split(/\0/, $values);
    my $value;
    $self->{display} = 0;
    foreach $value (@values)
    {
	$self->{display} += $value;
    }
}

sub setAllDisplay # set user display value to all possible values
{
    my $self = shift;

    my $value;
    $self->{display} = 0;
    foreach $value (values(%dispcodes))
    {
	$self->{display} += $value;
    }
}

sub isHidden # check wether a visual is displayed or not by the user
{
    my $self = shift;
    my $visual = shift; # visual to test

    if ($self->{display} & $dispcodes{$visual})
    {
	return ""; # "" if dislayed
    }
    else
    {
	return "hidden";
    }
}

sub getSubCondition # get SQL condition to select news from user subscribed templates
{
    my $self = shift;

    my $tmpls = $self->getSubTemplates();
    if ($tmpls && scalar(@{$tmpls}))
    {
	return "template in (".join(',', map {"'$_->{id}'"} @{$tmpls}).")";
    }
    else
    {
	return undef; # undef if no subscription
    }
}

sub getUnsubCondition # get SQL condition to select news from user unsubscribed templates
{
    my $self = shift;

    my $tmpls = $self->getUnsubTemplates();
    if ($tmpls && scalar(@{$tmpls}))
    {
	return "template in (".join(',', map {"'$_->{id}'"} @{$tmpls}).")";
    }
    else
    {
	return undef; # undef if no subscription
    }
}

sub nppSelectorCode # return HTML code for the display selector
{
    my $self = shift;
    my $submit = shift; # 1 to submit on value change
    my $name = shift || "newsperpage"; # optional selector name ("newsperpage" by default)

    my $code = "";
    $code .= '<select name="'.$name.'"';
    if ($submit)
    {
	$code .= ' onchange="form.submit();"';
    }
    $code .= ">\n";
    my $i;
    for ($i = 1; $i < 21; $i++)
    {
	$code .= '<option value="'.$i.'"';
	if ($self->{newsperpage} == $i)
	{
	    $code .= " selected";
	}
	$code .= '>'.$i."\n";
    }
    $code .= "</select>\n";

    return $code;
}
sub getLongName # return user long name
{
    my $self = shift;
    my $mail = shift; # 1 to add email address

    my $str = "";
    my $sep = "";
    if ($self->{firstname})
    {
	$str .= $sep.$self->{firstname};
	$sep = " ";
    }
    if ($self->{lastname})
    {
	$str .= $sep.$self->{lastname};
	$sep = " ";
    }
    if ($mail && $self->{mail})
    {
	$str .= $sep."<".$self->{mail}.">";
	$sep = " ";
    }
    if (!$str) { $str = $self->{login}; }

    return $str; # long name string
}

sub pwcrypt # return encryted string 
{
    my $self = shift;
    my $string = shift; # string to encrypt

    return crypt($string, rand(100)); # encryted string
}

sub setPassword # set the user password
{
    my $self = shift;
    my $pass = shift; # password to set

    $self->{password} = $self->pwcrypt($pass);
}

sub checkPassword # check the user password
{
    my $self = shift;
    my $pass = shift; # password to use to check

    my $salt = substr($self->{password}, 0, 2);
    my $strpass = crypt($pass, $salt);
    return ($strpass eq $self->{password});
}

sub getExtraVersion # get version of extra modules
{
    my $self = shift;

    $self->{extraversion} = "";
    $self->extrameths("getExtraVersion");
}

1;

__END__

=pod

=head1 NAME

MN::User - MyNews class to manage users.

=head1 SYNOPSIS

 use strict;
 use MN 1.0;

 my $mne = new MN::Engine();
 my $cgi = new MN::CGI();
 my $user = $cgi->getUser();
 ...

=head1 DESCRIPTION

MyNews User objects store informations about connected users. The current user is set only if the web server authentication is active (private area).

MN::User inherits from MN::Object, that means that user objects are stored in database.

=over 4

=head2 Fields

=item login : string

User login entered when user is autheticated. B<login> is the primary key of the MN::User class.

=item password : string

User password entered when user is autheticated (encrypted).

=item mail : string

User email address. This address is used to inform user when recent news has been added.

=item lang : string

Language used to display MyNews interface when user is connected.

=item theme : string

Graphical theme used to display MyNews interface when user is connected.

=item firstname : string

User first name. For display purpose only.

=item lastname : string

User last name. For display purpose only.

=item display : number

Bit field to manage the visibility of some elements (navigation bar, time, flag...).

=item newsperpage : number

Number of news displayed in each page for the connected user.

=head2 Subscriptions

=item getSubTemplates ([$ignoreactive])

Return the list of active templates the user has subscribed to. If $ignoreactive is set to 1, the list also contains inactive templates.

=item getUnsubTemplates ()

Return the list of active templates the user hasn't subscribed to.

=item getSubCondition ()

Return the SQL condition to select news from active templates the user has subscribed to.

=item getUnsubCondition ()

Return the SQL condition to select news from active templates the user hasn't subscribed to.

=head2 Admin

=item adminSelectorCode ([$name])

Return HTML code to display the user admin selector. If the user has admin privilege, the "Yes" option is selected. $name is the alternate field name of the selector ("admin" by default).

=item isAdmin ()

Return (i18n) "Yes" if user has admin privilege, "No" in the other case.

=item setAdmin ($value)

=head2 Display

=item adminSelectorCode ([$name])

Return HTML code to display the user display selector. The selector is a list of elements (navigation bar, time, flag...) selected or not depending on the value of the user B<display> field.

=item setDisplay ([$value1], [$value2]...)

Set the value of the user B<display> field. The value is the sum of all value parameters.

=item setAllDisplay ()

Set the value of the user B<display> field to the maximum (all bit to 1).

=item isHidden ($visual)

Return "hidden" if a visual is hidden for the user.

=head2 Misc

=item nppSelectorCode ([$submit], [$name])

Return HTML code to display the user news per page selector. The selected value is the value of the user B<newsperpage> field. If $submit = 1, the current form that contains the news per page selector is submited when the selected news per page is changed. $name is the alternate parameter name of the selector ("newsperpage" by default).

=item getLongName ([$mail])

Return the user long name (first name and last name). If $mail is set to 1, user email is added to long name.

=head1 SEE ALSO

L<MN::Object>, L<MN::Subscr>, L<MN::UserGroup>

=cut
